﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace VistaShot {
	/// <summary>
	/// Provides methods to 
	/// </summary>
	static class AlphaBitmapCalculator {

		/// <summary>
		/// Returns the pixels in a bitmap as a sequence of RGB pixel values, ignoring the padding from the stride of the image.
		/// </summary>
		/// <param name="bitmap">The bitmap to retrieve the pixels from.</param>
		/// <returns>A linear array of RGB pixel values, ignoring the padding from the stride of the image.</returns>
		static int[] GetPixelsFromBitmap(Bitmap bitmap) {
			// Create an array to store the result.
			int width = bitmap.Width, height = bitmap.Height;
			var result = new int[width * height];
			// Lock the image for reading.
			var lockedBitmap = bitmap.LockBits(new Rectangle(0, 0, width, height), ImageLockMode.ReadOnly, PixelFormat.Format32bppRgb);
			// Copy the data out.
			if (lockedBitmap.Stride == width * 4) {
				// When the stride matches the image width, just copy directly.
				Marshal.Copy(lockedBitmap.Scan0, result, 0, result.Length);
			} else {
				// Copy row by row.
				int sp = 0, dp = 0;
				for (int y = 0; y < height; ++y) {
					for (int x = 0; x < width; ++x) {
						result[dp++] = Marshal.ReadInt32(lockedBitmap.Scan0, sp++);
					}
					sp += (lockedBitmap.Stride / 4 - width);
				}
			}
			// Unlock source and return.
			bitmap.UnlockBits(lockedBitmap);
			return result;
		}

		/// <summary>
		/// Calculate the alpha channel from two images, one taken against a black background and the other taken against a white background.
		/// </summary>
		/// <param name="black">The image taken against a black background.</param>
		/// <param name="white">The image taken against a white background.</param>
		/// <returns>The bitmap with its restored alpha channel.</returns>
		public static Bitmap Calculate(Bitmap black, Bitmap white) {
			// Check both images are the same size.
			if (white.Width != black.Width || white.Height != black.Height) throw new ArgumentException("Both images must be the same size.");
			// Create a new image.
			int width = black.Width, height = black.Height;
			var result = new Bitmap(width, height, PixelFormat.Format32bppArgb);

			// Get the data for the two images.
			int[] blackPixels = GetPixelsFromBitmap(black), whitePixels = GetPixelsFromBitmap(white);
			
			// Lock the result for writing.
			var lockedResult = result.LockBits(new Rectangle(0, 0, width, height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
			var resultPixels = new int[height * lockedResult.Stride / 4];
			
			// Process the two images to calculate the result.
			int sp = 0, dp = 0;
			for (int y = 0; y < width; ++y) {
				for (int x = 0; x < height; ++x) {
					// Fetch the pixel to process.
					int blackPixel = blackPixels[sp], whitePixel = whitePixels[sp++];
					// Calculate the alpha from the difference between the black and white images.
					byte alpha = (byte)((byte)blackPixel - (byte)whitePixel + 255);
					// Use this alpha to restore the RGB components.
					byte r = 0, g = 0, b = 0;
					if (alpha != 0) {
						b = (byte)(((blackPixel & 0xFF) * 255) / alpha);
						g = (byte)((((blackPixel >> 8) & 0xFF) * 255) / alpha);
						r = (byte)((((blackPixel >> 16) & 0xFF) * 255) / alpha);
					}
					// Write the resulting pixel.
					resultPixels[dp++] = b + (g << 8) + (r << 16) + (alpha << 24);
				}
				dp += (lockedResult.Stride / 4 - width);
			}
			
			// Copy pixels, unlock result and return.
			Marshal.Copy(resultPixels, 0, lockedResult.Scan0, resultPixels.Length);
			result.UnlockBits(lockedResult);
			return result;
		}
	}
}
