﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using System.Media;

namespace VistaShot {
	static class Program {

		#region Win32

		/// <summary>
		/// Gets a handle to the current foreground window.
		/// </summary>
		/// <returns>The handle of the current foreground window, or zero if there isn't one.</returns>
		[DllImport("user32.dll")]
		static extern IntPtr GetForegroundWindow();

		/// <summary>
		/// Sets the foreground window from its handle.
		/// </summary>
		/// <param name="hWnd">The handle of the window to make the foreground window.</param>
		/// <returns>True if the window was sent to the foreground, false otherwise.</returns>
		[DllImport("user32.dll")]
		static extern bool SetForegroundWindow(IntPtr hWnd);

		/// <summary>
		/// Gets the position and size of a window on the screen.
		/// </summary>
		/// <param name="hWnd">The handle of the window to find the position and size of.</param>
		/// <param name="lpRect">Returns the position and size of the window.</param>
		/// <returns>True if the position and size could be found, false otherwise.</returns>
		[DllImport("user32.dll")]
		static extern bool GetWindowRect(IntPtr hWnd, ref RECT lpRect);

		/// <summary>
		/// Represents a rectangle.
		/// </summary>
		struct RECT {
			/// <summary>
			/// Gets or sets the left coordinate of the rectangle.
			/// </summary>
			public int left;
			/// <summary>
			/// Gets or sets the top coordinate of the rectangle.
			/// </summary>
			public int top;
			/// <summary>
			/// Gets or sets the right coordinate of the rectangle.
			/// </summary>
			public int right;
			/// <summary>
			/// Gets or sets the bottom coordinate of the rectangle.
			/// </summary>
			public int bottom;
			/// <summary>
			/// Gets the width of the rectangle.
			/// </summary>
			public int width { get { return this.right - this.left; } }
			/// <summary>
			/// Gets the height of the rectangle.
			/// </summary>
			public int height { get { return this.bottom - this.top; } }
		}

		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {

			// Timer:
			for (int i = 0; i < 5; ++i) {
				SystemSounds.Beep.Play();
				Thread.Sleep(1000);
			}

			try {

				// Find the window to take a screenshot of.
				var windowToScreenshot = GetForegroundWindow();
				if (windowToScreenshot == IntPtr.Zero) throw new Exception("There is no window to take a screenshot of.");

				// Calculate the size of the window rectangle.
				var windowRect = new RECT();
				if (!GetWindowRect(windowToScreenshot, ref windowRect)) throw new Exception("Could not retrieve the position and size of the window to screenshot.");

				// Apply padding around the rectangle to accomodate the shadow.
				int shadowPadding = 20; // Magic number?
				windowRect.left -= shadowPadding;
				windowRect.right += shadowPadding;
				windowRect.top -= shadowPadding;
				windowRect.bottom += shadowPadding;

				// Create a pair of target bitmaps.
				using (var black = new Bitmap(windowRect.width, windowRect.height))
				using (var white = new Bitmap(windowRect.width, windowRect.height)) {

					// Use a special form to force a particular background colour.
					using (var forcedBackgroundForm = new Form() {
						Left = windowRect.left,
						Top = windowRect.top,
						ClientSize = new Size(windowRect.width, windowRect.height),
						StartPosition = FormStartPosition.Manual,
						FormBorderStyle = FormBorderStyle.None,
						BackColor = Color.Black,
						ShowInTaskbar = false,
					}) {
						// Show the "forced background" window.
						forcedBackgroundForm.Show();
						forcedBackgroundForm.BringToFront();

						SetForegroundWindow(windowToScreenshot);
						Thread.Sleep(100);
						Application.DoEvents();

						// Take a black screenshot against this window.
						using (var g = Graphics.FromImage(black)) {
							g.CopyFromScreen(windowRect.left, windowRect.top, 0, 0, new Size(windowRect.width, windowRect.height));
						}

						// Set the background window to white.
						forcedBackgroundForm.BackColor = Color.White;
						Thread.Sleep(100);
						Application.DoEvents();

						// Take a white screenshot against this window.
						using (var g = Graphics.FromImage(white)) {
							g.CopyFromScreen(windowRect.left, windowRect.top, 0, 0, new Size(windowRect.width, windowRect.height));
						}

						// Remove the special form.
						forcedBackgroundForm.Close();
					}

					// Convert the black/white images into an alpha image, then save.
					using (var alpha = AlphaBitmapCalculator.Calculate(black, white))
					using (var saveDialog = new SaveFileDialog() {
						Filter = "PNG (*.png)|*.png",
					}) {
						if (saveDialog.ShowDialog() == DialogResult.OK) {
							try {
								alpha.Save(saveDialog.FileName);
							} catch (Exception ex) {
								MessageBox.Show("Could not save image: " + ex.Message, Application.ProductName);
							}
						}
					}
				}
			
			} catch (Exception ex) {
				MessageBox.Show("Error taking screenshot: " + ex.Message, Application.ProductName);
			}

		}
	}
}
